@@ -1,5 +1,5 @@
 /* Extract some translations of a translation catalog.
-   Copyright (C) 2001-2003 Free Software Foundation, Inc.
+   Copyright (C) 2001-2004 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software; you can redistribute it and/or modify
@@ -51,6 +51,7 @@
 #include "str-list.h"
 #include "msgl-charset.h"
 #include "xalloc.h"
+#include "xallocsa.h"
 #include "exit.h"
 #include "full-write.h"
 #include "findprog.h"
@@ -314,7 +315,7 @@ main (int argc, char **argv)
 This is free software; see the source for copying conditions.  There is NO\n\
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
 "),
-	      "2001-2003");
+	      "2001-2004");
       printf (_("Written by %s.\n"), "Bruno Haible");
       exit (EXIT_SUCCESS);
     }
@@ -604,7 +605,7 @@ is_string_selected (int grep_pass, const char *str, size_t len)
       close (fd[0]);
 
       /* Remove zombie process from process list, and retrieve exit status.  */
-      exitstatus = wait_subprocess (child, "grep", false, true, true);
+      exitstatus = wait_subprocess (child, "grep", false, false, true, true);
       return (exitstatus == 0);
     }
   else
@@ -659,11 +660,12 @@ is_message_selected (const message_ty *mp)
       char *total_comment;
       char *q;
       size_t j;
+      bool selected;
 
       length = 0;
       for (j = 0; j < mp->comment->nitems; j++)
 	length += strlen (mp->comment->item[j]) + 1;
-      total_comment = (char *) alloca (length);
+      total_comment = (char *) xallocsa (length);
 
       q = total_comment;
       for (j = 0; j < mp->comment->nitems; j++)
@@ -677,7 +679,11 @@ is_message_selected (const message_ty *mp)
       if (q != total_comment + length)
 	abort ();
 
-      if (is_string_selected (2, total_comment, length))
+      selected = is_string_selected (2, total_comment, length);
+
+      freesa (total_comment);
+
+      if (selected)
 	return true;
     }
 
