@@ -1,5 +1,5 @@
 /* msgunfmt - converts binary .mo files to Uniforum style .po files
-   Copyright (C) 1995-1998, 2000-2003 Free Software Foundation, Inc.
+   Copyright (C) 1995-1998, 2000-2004 Free Software Foundation, Inc.
    Written by Ulrich Drepper <drepper@gnu.ai.mit.edu>, April 1995.
 
    This program is free software; you can redistribute it and/or modify
@@ -38,6 +38,8 @@
 #include "msgunfmt.h"
 #include "read-mo.h"
 #include "read-java.h"
+#include "read-csharp.h"
+#include "read-resources.h"
 #include "read-tcl.h"
 #include "write-po.h"
 #include "gettext.h"
@@ -53,6 +55,15 @@ static bool java_mode;
 static const char *java_resource_name;
 static const char *java_locale_name;
 
+/* C# mode input file specification.  */
+static bool csharp_mode;
+static const char *csharp_resource_name;
+static const char *csharp_locale_name;
+static const char *csharp_base_directory;
+
+/* C# resources mode input file specification.  */
+static bool csharp_resources_mode;
+
 /* Tcl mode input file specification.  */
 static bool tcl_mode;
 static const char *tcl_locale_name;
@@ -64,6 +75,8 @@ static int force_po;
 /* Long options.  */
 static const struct option long_options[] =
 {
+  { "csharp", no_argument, NULL, CHAR_MAX + 4 },
+  { "csharp-resources", no_argument, NULL, CHAR_MAX + 5 },
   { "escape", no_argument, NULL, 'E' },
   { "force-po", no_argument, &force_po, 1 },
   { "help", no_argument, NULL, 'h' },
@@ -92,6 +105,7 @@ static void usage (int status)
 	__attribute__ ((noreturn))
 #endif
 ;
+static void read_one_file (message_list_ty *mlp, const char *filename);
 
 
 int
@@ -130,6 +144,7 @@ main (int argc, char **argv)
 	break;
 
       case 'd':
+	csharp_base_directory = optarg;
 	tcl_base_directory = optarg;
 	break;
 
@@ -155,6 +170,7 @@ main (int argc, char **argv)
 
       case 'l':
 	java_locale_name = optarg;
+	csharp_locale_name = optarg;
 	tcl_locale_name = optarg;
 	break;
 
@@ -168,6 +184,7 @@ main (int argc, char **argv)
 
       case 'r':
 	java_resource_name = optarg;
+	csharp_resource_name = optarg;
 	break;
 
       case 's':
@@ -196,7 +213,7 @@ main (int argc, char **argv)
 	}
 	break;
 
-      case CHAR_MAX + 1:
+      case CHAR_MAX + 1: /* --tcl */
 	tcl_mode = true;
 	break;
 
@@ -208,6 +225,14 @@ main (int argc, char **argv)
 	message_print_syntax_stringtable ();
 	break;
 
+      case CHAR_MAX + 4: /* --csharp */
+	csharp_mode = true;
+	break;
+
+      case CHAR_MAX + 5: /* --csharp-resources */
+	csharp_resources_mode = true;
+	break;
+
       default:
 	usage (EXIT_FAILURE);
 	break;
@@ -222,7 +247,7 @@ main (int argc, char **argv)
 This is free software; see the source for copying conditions.  There is NO\n\
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
 "),
-	      "1995-1998, 2000-2003");
+	      "1995-1998, 2000-2004");
       printf (_("Written by %s.\n"), "Ulrich Drepper");
       exit (EXIT_SUCCESS);
     }
@@ -232,9 +257,32 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
     usage (EXIT_SUCCESS);
 
   /* Check for contradicting options.  */
-  if (java_mode && tcl_mode)
-    error (EXIT_FAILURE, 0, _("%s and %s are mutually exclusive"),
-	   "--java", "--tcl");
+  {
+    unsigned int modes =
+      (java_mode ? 1 : 0)
+      | (csharp_mode ? 2 : 0)
+      | (csharp_resources_mode ? 4 : 0)
+      | (tcl_mode ? 8 : 0);
+    static const char *mode_options[] =
+      { "--java", "--csharp", "--csharp-resources", "--tcl" };
+    /* More than one bit set?  */
+    if (modes & (modes - 1))
+      {
+	const char *first_option;
+	const char *second_option;
+	unsigned int i;
+	for (i = 0; ; i++)
+	  if (modes & (1 << i))
+	    break;
+	first_option = mode_options[i];
+	for (i = i + 1; ; i++)
+	  if (modes & (1 << i))
+	    break;
+	second_option = mode_options[i];
+	error (EXIT_FAILURE, 0, _("%s and %s are mutually exclusive"),
+	       first_option, second_option);
+      }
+  }
   if (java_mode)
     {
       if (optind < argc)
@@ -244,6 +292,29 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
 		 "--java");
 	}
     }
+  else if (csharp_mode)
+    {
+      if (optind < argc)
+	{
+	  error (EXIT_FAILURE, 0,
+		 _("%s and explicit file names are mutually exclusive"),
+		 "--csharp");
+	}
+      if (csharp_locale_name == NULL)
+	{
+	  error (EXIT_SUCCESS, 0,
+		 _("%s requires a \"-l locale\" specification"),
+		 "--csharp");
+	  usage (EXIT_FAILURE);
+	}
+      if (csharp_base_directory == NULL)
+	{
+	  error (EXIT_SUCCESS, 0,
+		 _("%s requires a \"-d directory\" specification"),
+		 "--csharp");
+	  usage (EXIT_FAILURE);
+	}
+    }
   else if (tcl_mode)
     {
       if (optind < argc)
@@ -271,14 +342,14 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
     {
       if (java_resource_name != NULL)
 	{
-	  error (EXIT_SUCCESS, 0, _("%s is only valid with %s"),
-		 "--resource", "--java");
+	  error (EXIT_SUCCESS, 0, _("%s is only valid with %s or %s"),
+		 "--resource", "--java", "--csharp");
 	  usage (EXIT_FAILURE);
 	}
       if (java_locale_name != NULL)
 	{
-	  error (EXIT_SUCCESS, 0, _("%s is only valid with %s"),
-		 "--locale", "--java");
+	  error (EXIT_SUCCESS, 0, _("%s is only valid with %s or %s"),
+		 "--locale", "--java", "--csharp");
 	  usage (EXIT_FAILURE);
 	}
     }
@@ -288,6 +359,11 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
     {
       result = msgdomain_read_java (java_resource_name, java_locale_name);
     }
+  else if (csharp_mode)
+    {
+      result = msgdomain_read_csharp (csharp_resource_name, csharp_locale_name,
+				      csharp_base_directory);
+    }
   else if (tcl_mode)
     {
       result = msgdomain_read_tcl (tcl_locale_name, tcl_base_directory);
@@ -300,11 +376,11 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
       if (optind < argc)
 	{
 	  do
-	    read_mo_file (mlp, argv[optind]);
+	    read_one_file (mlp, argv[optind]);
 	  while (++optind < argc);
 	}
       else
-	read_mo_file (mlp, "-");
+	read_one_file (mlp, "-");
 
       result = msgdomain_list_alloc (false);
       result->item[0]->messages = mlp;
@@ -346,6 +422,10 @@ Mandatory arguments to long options are mandatory for short options too.\n"));
 Operation mode:\n"));
       printf (_("\
   -j, --java                  Java mode: input is a Java ResourceBundle class\n"));
+      printf (_("\
+      --csharp                C# mode: input is a .NET .dll file\n"));
+      printf (_("\
+      --csharp-resources      C# resources mode: input is a .NET .resources file\n"));
       printf (_("\
       --tcl                   Tcl mode: input is a tcl/msgcat .msg file\n"));
       printf ("\n");
@@ -368,6 +448,18 @@ separated with an underscore.  The class is located using the CLASSPATH.\n\
 "));
       printf ("\n");
       printf (_("\
+Input file location in C# mode:\n"));
+      printf (_("\
+  -r, --resource=RESOURCE     resource name\n"));
+      printf (_("\
+  -l, --locale=LOCALE         locale name, either language or language_COUNTRY\n"));
+      printf (_("\
+  -d DIRECTORY                base directory for locale dependent .dll files\n"));
+      printf (_("\
+The -l and -d options are mandatory.  The .dll file is located in a\n\
+subdirectory of the specified directory whose name depends on the locale.\n"));
+      printf ("\n");
+      printf (_("\
 Input file location in Tcl mode:\n"));
       printf (_("\
   -l, --locale=LOCALE         locale name, either language or language_COUNTRY\n"));
@@ -424,3 +516,13 @@ Informative output:\n"));
 
   exit (status);
 }
+
+
+static void
+read_one_file (message_list_ty *mlp, const char *filename)
+{
+  if (csharp_resources_mode)
+    read_resources_file (mlp, filename);
+  else
+    read_mo_file (mlp, filename);
+}
