@@ -78,6 +78,7 @@
 #include "plural-exp.h"
 #include "po-charset.h"
 #include "xalloc.h"
+#include "xallocsa.h"
 #include "pathname.h"
 #include "fatal-signal.h"
 #include "fwriteerror.h"
@@ -158,7 +159,8 @@ compute_hashsize (message_list_ty *mlp, bool *collisionp)
 #define XXN 3  /* can be tweaked */
 #define XXS 3  /* can be tweaked */
   unsigned int n = mlp->nitems;
-  unsigned int *hashcodes = (unsigned int *) alloca (n * sizeof (unsigned int));
+  unsigned int *hashcodes =
+    (unsigned int *) xallocsa (n * sizeof (unsigned int));
   unsigned int hashsize;
   unsigned int best_hashsize;
   unsigned int best_score;
@@ -275,6 +277,8 @@ compute_hashsize (message_list_ty *mlp, bool *collisionp)
   if (best_hashsize == 0 || best_score < best_hashsize)
     abort ();
 
+  freesa (hashcodes);
+
   /* There are collisions if and only if best_score > best_hashsize.  */
   *collisionp = (best_score > best_hashsize);
   return best_hashsize;
@@ -738,7 +742,8 @@ write_java_code (FILE *stream, const char *class_name, message_list_ty *mlp,
       if (plurals)
 	{
 	  bool first;
-	  fprintf (stream, "  public static final java.lang.String[] plural = new java.lang.String[] { ");
+	  fprintf (stream, "  public static final java.lang.String[] get_msgid_plural_table () {\n");
+	  fprintf (stream, "    return new java.lang.String[] { ");
 	  first = true;
 	  for (j = 0; j < mlp->nitems; j++)
 	    {
@@ -752,6 +757,7 @@ write_java_code (FILE *stream, const char *class_name, message_list_ty *mlp,
 		}
 	    }
 	  fprintf (stream, " };\n");
+	  fprintf (stream, "  }\n");
 	}
 
       if (plurals)
@@ -817,8 +823,7 @@ write_java_code (FILE *stream, const char *class_name, message_list_ty *mlp,
       /* Emit the msgid_plural strings.  Only used by msgunfmt.  */
       if (plurals)
 	{
-	  fprintf (stream, "  public static final java.util.Hashtable plural;\n");
-	  fprintf (stream, "  static {\n");
+	  fprintf (stream, "  public static final java.util.Hashtable get_msgid_plural_table () {\n");
 	  fprintf (stream, "    java.util.Hashtable p = new java.util.Hashtable();\n");
 	  for (j = 0; j < mlp->nitems; j++)
 	    if (mlp->item[j]->msgid_plural != NULL)
@@ -829,7 +834,7 @@ write_java_code (FILE *stream, const char *class_name, message_list_ty *mlp,
 		write_java_string (stream, mlp->item[j]->msgid_plural);
 		fprintf (stream, ");\n");
 	      }
-	  fprintf (stream, "    plural = p;\n");
+	  fprintf (stream, "    return p;\n");
 	  fprintf (stream, "  }\n");
 	}
 
@@ -970,7 +975,7 @@ msgdomain_write_java (message_list_ty *mlp, const char *canon_encoding,
   }
 
   /* Create a temporary directory where we can put the Java file.  */
-  template = (char *) alloca (PATH_MAX);
+  template = (char *) xallocsa (PATH_MAX);
   if (path_search (template, PATH_MAX, NULL, "msg", 1))
     {
       error (0, errno,
@@ -1010,7 +1015,7 @@ msgdomain_write_java (message_list_ty *mlp, const char *canon_encoding,
   else
     class_name = xstrdup (resource_name);
 
-  subdirs = (ndots > 0 ? (char **) alloca (ndots * sizeof (char *)) : NULL);
+  subdirs = (ndots > 0 ? (char **) xallocsa (ndots * sizeof (char *)) : NULL);
   {
     const char *p;
     const char *last_dir;
@@ -1022,10 +1027,11 @@ msgdomain_write_java (message_list_ty *mlp, const char *canon_encoding,
       {
 	const char *q = strchr (p, '.');
 	size_t n = q - p;
-	char *part = (char *) alloca (n + 1);
+	char *part = (char *) xallocsa (n + 1);
 	memcpy (part, p, n);
 	part[n] = '\0';
 	subdirs[i] = concatenated_pathname (last_dir, part, NULL);
+	freesa (part);
 	last_dir = subdirs[i];
 	p = q + 1;
       }
@@ -1113,11 +1119,13 @@ compilation of Java class failed, please try --verbose or set $JAVAC"));
     for (i = 0; i < ndots; i++)
       free (subdirs[i]);
   }
+  freesa (subdirs);
   free (class_name);
  quit2:
   rmdir (tmpdir);
  quit1:
   cleanup_list.tmpdir = NULL;
+  freesa (template);
   /* Here we could unregister the cleanup() handler.  */
   return retval;
 }
