@@ -34,8 +34,10 @@
 #include "message.h"
 #include "msgl-iconv.h"
 #include "po-charset.h"
-#include "xmalloc.h"
+#include "xalloc.h"
+#include "xallocsa.h"
 #include "pathname.h"
+#include "fwriteerror.h"
 #include "exit.h"
 #include "utf8-ucs4.h"
 #include "gettext.h"
@@ -173,7 +175,7 @@ but the Tcl message catalog format doesn't support plural handling\n")));
 
     /* Convert the locale name to lowercase and remove any encoding.  */
     len = strlen (locale_name);
-    frobbed_locale_name = (char *) alloca (len + 1);
+    frobbed_locale_name = (char *) xallocsa (len + 1);
     memcpy (frobbed_locale_name, locale_name, len + 1);
     for (p = frobbed_locale_name; *p != '\0'; p++)
       if (*p >= 'A' && *p <= 'Z')
@@ -185,23 +187,25 @@ but the Tcl message catalog format doesn't support plural handling\n")));
 	}
 
     file_name = concatenated_pathname (directory, frobbed_locale_name, ".msg");
-      
+
     output_file = fopen (file_name, "w");
     if (output_file == NULL)
       {
 	error (0, errno, _("error while opening \"%s\" for writing"),
 	       file_name);
+	freesa (frobbed_locale_name);
 	return 1;
       }
 
     write_msg (output_file, mlp, frobbed_locale_name);
 
     /* Make sure nothing went wrong.  */
-    if (fflush (output_file) || ferror (output_file))
+    if (fwriteerror (output_file))
       error (EXIT_FAILURE, errno, _("error while writing \"%s\" file"),
 	     file_name);
 
     fclose (output_file);
+    freesa (frobbed_locale_name);
   }
 
   return 0;
