@@ -43,6 +43,10 @@
 
 #define _(s) gettext(s)
 
+#define max(a,b) ((a) > (b) ? (a) : (b))
+
+#define SIZEOF(a) (sizeof(a) / sizeof(a[0]))
+
 
 /* The Python syntax is defined in the Python Reference Manual
    /usr/share/doc/packages/python/html/ref/index.html.
@@ -144,10 +148,9 @@ static int line_number;
 static FILE *fp;
 
 
-/* 1. line_number handling.  Also allow a lookahead of 9 characters.  */
+/* 1. line_number handling.  Also allow a lookahead.  */
 
-/* Maximum used guaranteed to be < UNINAME_MAX + 4.  */
-static unsigned char phase1_pushback[UNINAME_MAX + 4];
+static unsigned char phase1_pushback[max (9, UNINAME_MAX + 3)];
 static int phase1_pushback_length;
 
 static int
@@ -176,6 +179,7 @@ phase1_getc ()
   return c;
 }
 
+/* Supports max (9, UNINAME_MAX + 3) characters of pushback.  */
 static void
 phase1_ungetc (int c)
 {
@@ -184,6 +188,8 @@ phase1_ungetc (int c)
       if (c == '\n')
 	--line_number;
 
+      if (phase1_pushback_length == SIZEOF (phase1_pushback))
+	abort ();
       phase1_pushback[phase1_pushback_length++] = c;
     }
 }
@@ -234,7 +240,7 @@ comment_line_end ()
       buffer = xrealloc (buffer, bufmax);
     }
   buffer[buflen] = '\0';
-  xgettext_comment_add (buffer);
+  savable_comment_add (buffer);
 }
 
 /* These are for tracking whether comments count as immediately before
@@ -288,6 +294,7 @@ phase2_getc ()
     }
 }
 
+/* Supports only one pushback character.  */
 static void
 phase2_ungetc (int c)
 {
@@ -315,6 +322,7 @@ struct token_ty
 {
   token_type_ty type;
   char *string;		/* for token_type_string, token_type_symbol */
+  refcounted_string_list_ty *comment;	/* for token_type_string */
   int line_number;
 };
 
@@ -656,8 +664,7 @@ phase7_getuc (int quote_char,
 /* Number of pending open parentheses/braces/brackets.  */
 static int open_pbb;
 
-/* Maximum used guaranteed to be < 2.  */
-static token_ty phase5_pushback[2];
+static token_ty phase5_pushback[1];
 static int phase5_pushback_length;
 
 static void
@@ -690,7 +697,7 @@ phase5_get (token_ty *tp)
 
 	case '\n':
 	  if (last_non_comment_line > last_comment_line)
-	    xgettext_comment_reset ();
+	    savable_comment_reset ();
 	  /* Ignore newline if and only if it is used for implicit line
 	     joining.  */
 	  if (open_pbb > 0)
@@ -905,6 +912,7 @@ phase5_get (token_ty *tp)
 		assert (q - utf8_string <= 3 * bufpos);
 		tp->string = (char *) utf8_string;
 	      }
+	      tp->comment = add_reference (savable_comment);
 	      tp->type = token_type_string;
 	      return;
 	  }
@@ -945,11 +953,16 @@ phase5_get (token_ty *tp)
     }
 }
 
+/* Supports only one pushback token.  */
 static void
 phase5_unget (token_ty *tp)
 {
   if (tp->type != token_type_eof)
-    phase5_pushback[phase5_pushback_length++] = *tp;
+    {
+      if (phase5_pushback_length == SIZEOF (phase5_pushback))
+	abort ();
+      phase5_pushback[phase5_pushback_length++] = *tp;
+    }
 }
 
 
@@ -1110,7 +1123,11 @@ extract_parenthesized (message_list_ty *mlp,
 	    pos.line_number = token.line_number;
 
 	    if (extract_all)
-	      remember_a_message (mlp, token.string, inner_context, &pos);
+	      {
+		savable_comment_to_xgettext_comment (token.comment);
+		remember_a_message (mlp, token.string, inner_context, &pos);
+		savable_comment_reset ();
+	      }
 	    else
 	      {
 		if (commas_to_skip == 0)
@@ -1118,9 +1135,12 @@ extract_parenthesized (message_list_ty *mlp,
 		    if (plural_mp == NULL)
 		      {
 			/* Seen an msgid.  */
-			message_ty *mp =
-			  remember_a_message (mlp, token.string,
-					      inner_context, &pos);
+			message_ty *mp;
+
+			savable_comment_to_xgettext_comment (token.comment);
+			mp = remember_a_message (mlp, token.string,
+						inner_context, &pos);
+			savable_comment_reset ();
 			if (plural_commas > 0)
 			  plural_mp = mp;
 		      }
@@ -1136,6 +1156,7 @@ extract_parenthesized (message_list_ty *mlp,
 		  free (token.string);
 	      }
 	  }
+	  drop_reference (token.comment);
 	  next_context_iter = null_context_list_iterator;
 	  state = 0;
 	  continue;
